@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 require './spec/cases/helper'
 
 Signal.trap :SIGINT do
@@ -6,8 +7,7 @@ Signal.trap :SIGINT do
   exit 0
 end
 
-Parallel.map(Array.new(20), :in_processes => 2) do
+Parallel.map(Array.new(20), in_processes: 2) do
   sleep 10
   puts "I should be killed earlier"
 end
-
