@@ -1,7 +1,8 @@
+# frozen_string_literal: true
 require './spec/cases/helper'
 
 begin
-  Parallel.map([1,2,3]) do |x, i|
+  Parallel.map([1, 2, 3]) do |_x, _i|
     Process.kill("SIGKILL", Process.pid)
   end
 rescue Parallel::DeadWorker
